#!/bin/bash

# checkvg.sh
#
# Copyright (c) 2013, 2014, Oracle and/or its affiliates. All rights reserved.
#
#    NAME
#      checkvg.sh
#
#    NOTES


#--------------------------------------------------------------------------------
# PROCEDURE    : PrintMsg
# INPUT        : $1: log the message (log/nolog)
#              : $2: state (error/info/warning)
#              : $3: message (string)
# DESCRIPTION  : Print output in a similar formatted way. Needs further output formatting
#------------------------------------------------------------------------------
PrintMsg()
{
   local v_log="${1}"
   local v_state="${2}"
   local v_mesg="${3}"

   case "${v_state}" in
         error)
                echo "  (*) - ERROR: "${v_mesg}""
                if [ "${v_log}" == "log" ]
                then
                   imageLogger_logMsg $lvmchecker $imageLogger_LOG_ERROR "${v_logger_msg}"
                fi
                ;;
         info)
                echo "  (*) "${v_mesg}""
                if [ "${v_log}" == "log" ]
                then
                   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
                fi
                ;;
         warning)
                echo "  (*) - WARNING: "${v_mesg}""
                if [ "${v_log}" == "log" ]
                then
                   imageLogger_logMsg $lvmchecker $imageLogger_LOG_WARNING "${v_logger_msg}"
                fi
                ;;
         esac
}

#--------------------------------------------------------------------------------
# PROCEDURE    : usage
# INPUT        :
# DESCRIPTION  : Print the options/flags the user can use with an example. eeds further finishing
#------------------------------------------------------------------------------
usage()
{
   echo
   PrintMsg log info  "Use this script as follows: ./${v_myname} checkall|vgfree|lvmactive|lvmsize|lvmmin|vgexa|islvm"
   echo
}

#--------------------------------------------------------------------------------
# PROCEDURE    : InitLog
# INPUT        :
# DESCRIPTION  :
#------------------------------------------------------------------------------
InitLog()
{
   v_logger_msg="Entering ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   v_logger_msg="zzz"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   v_logger_msg="#################################################################################"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   v_logger_msg="# ${v_myname} script rel. : ${v_myname_version} started at ${v_timestamp} (runid :${v_runid})"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   v_logger_msg="# arguments given         : ${v_arguments}"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   v_logger_msg="#################################################################################"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   v_logger_msg="Leaving ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
}

#--------------------------------------------------------------------------------
# PROCEDURE    : ExecAllChecks
# INPUT        :
# DESCRIPTION  : Execute all checks described in 19068757 
#------------------------------------------------------------------------------
ExecAllChecks()
{
   v_logger_msg="Entering ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   CheckIsLvmSys
   CheckVGExa
   CheckMinLvm
   CheckLvmSize
   CheckLvmActive
   CheckFreeSpaceVG

   v_logger_msg="Leaving ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
}

#--------------------------------------------------------------------------------
# PROCEDURE    : PrintErrorReturn
# INPUT        : $1: exit/noexit 
#                also uses glbal var. v_checks_failed and v_error_cnt
# DESCRIPTION  : return exit code and print out the error itself
#                only for non-lvm systems it should exit 1 immediately
#------------------------------------------------------------------------------
PrintErrorReturn()
{
   v_logger_msg="Entering ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   local v_error=""

   if [ "${v_checks_failed}" != "" ]
   then
      for v_error in "${v_checks_failed}"
      do
         v_logger_msg="Failed on ${v_error}"
         imageLogger_logMsg $lvmchecker $imageLogger_LOG_ERROR "${v_logger_msg}"
         case "${v_error}" in
             CheckIsLvmSys)
                    PrintMsg log error "This is system does not have lvm configured. System with lvm enabled have additional options and are recommended"
                    v_logger_msg="Command to validate: awk -F= '/^lvm=/ {print $2}' /opt/oracle.cellos/ORACLE_CELL_OS_IS_SETUP"
                    imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
                     ;;
             CheckVGExa)
                    PrintMsg log error "This is system does not have the expected volmegroup VgExaDB."
                    v_logger_msg="Command to validate: /usr/sbin/lvdisplay --ignorelockingfailure --columns --noheadings -o +vg_name,lv_name"
                    imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
                     ;;
             CheckMinLvm)
                    PrintMsg log error "This is system does not have the expected minimum number of sys lvms"
                    v_logger_msg="Command to validate: /usr/sbin/lvdisplay --ignorelockingfailure --columns --noheadings -o lv_name | grep " LVDbSys[1-2] "| wc -l"
                    imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
                     ;;
             CheckLvmSize)
                    PrintMsg log error "This is system does not have the  minimum and same size of 30G for the sys lvms"
                    v_logger_msg="Command to validate: lvdisplay --ignorelockingfailure --columns --noheadings -o +lv_name,lv_size --units g"
                    imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
                     ;;
             CheckFreeSpaceVG)
                    PrintMsg log error "Total amount of free space in the volume group is less than 2GB. Unable to make a snapshot"
                    v_logger_msg="Command to validate: vgdisplay VGExaDb --ignorelockingfailure --columns --noheadings -o vg_free"
                    imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
                     ;;
             CheckLvmActive)
                    PrintMsg log error "This is system has inactive lvms mounted which is not ok"
                    v_logger_msg="Command to validate: find active Sys lvm with imageinfo -sys. If active Sys lvm is Sys1, then Sys2 should not be mounted"
                    imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
                     ;;
         esac
      done

      # only for non-lvm systems it should exit 1 immediately
      # for others we accumulate
      if [ "${1}" == "exit" ]
      then
         exit 1
      else
         v_error_cnt=`expr ${v_error_cnt} + 1 2>/dev/null`
      fi
   fi

   v_logger_msg="Leaving ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
}

#--------------------------------------------------------------------------------
# PROCEDURE    : CheckMinLvm
# INPUT        :
# DESCRIPTION  : Count the minim number of Sys lvms. For systems running on Sys2 we expect a Sys1, which makes 2
#------------------------------------------------------------------------------
CheckMinLvm()
{
   v_logger_msg="Entering ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   local v_imageinfo="/usr/local/bin/imageinfo"
   local v_active_image=`${v_imageinfo} --system-partition 2>/dev/null`
   local -i v_tmp_cnt=0

   if [ "${v_active_image}" == "/dev/mapper/VGExaDb-LVDbSys1" ]
   then
      # We expect at least LVDbSys1
      v_tmp_cnt=`/usr/sbin/lvdisplay --ignorelockingfailure --columns --noheadings -o lv_name | grep " LVDbSys[1-2] "| wc -l`
      if [ ${v_tmp_cnt} -lt 1 ] 
      then
         v_checks_failed="CheckMinLvm"
         PrintErrorReturn noexit
      else
         v_logger_msg="PASS: Minimum number of Sys lvms"
         PrintMsg nolog info "${v_logger_msg}"
      fi
   fi

   if [ "${v_active_image}" == "/dev/mapper/VGExaDb-LVDbSys2" ]
   then
      # We expect at least LVDbSys1 and LVDbSys2 to exist
      v_tmp_cnt=`/usr/sbin/lvdisplay --ignorelockingfailure --columns --noheadings -o lv_name | grep " LVDbSys[1-2] "| wc -l`
      if [ ${v_tmp_cnt} -lt 2 ] 
      then
         v_checks_failed="CheckMinLvm"
         PrintErrorReturn noexit
      else
         v_logger_msg="PASS: minimum number of sys lvms"
         PrintMsg nolog info "${v_logger_msg}"
      fi
   fi

   v_logger_msg="Leaving ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
}

#--------------------------------------------------------------------------------
# PROCEDURE    : CheckLvmSize
# INPUT        :
# DESCRIPTION  : Check for a minimum size and for both sys lvms to be the same size
#------------------------------------------------------------------------------
CheckLvmSize()
{
   v_logger_msg="Entering ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   local v_lvm="/sbin/lvm"
   local v_dbvg="VGExaDb"
   local v_imageinfo="/usr/local/bin/imageinfo"
   local v_active_lvm_name=`${v_imageinfo} --system-partition 2>/dev/null`
   local -i v_tmp_cnt=0

   if [ "${v_active_lvm_name}" == "/dev/mapper/${v_dbvg}-LVDbSys1" ]
   then
      local v_inactive_lvm_name=/dev/mapper/${v_dbvg}-LVDbSys2
      local v_inactive_lvm_block="/dev/${v_dbvg}/LVDbSys2"
      local v_active_lvm_block="/dev/${v_dbvg}/LVDbSys1"
      local v_inactive_part_no=2
   fi

   if [ "${v_active_lvm_name}" == "/dev/mapper/${v_dbvg}-LVDbSys2" ]
   then
      v_inactive_lvm_name=/dev/mapper/${v_dbvg}-LVDbSys1
      v_inactive_lvm_block="/dev/${v_dbvg}/LVDbSys1"
      v_active_lvm_block="/dev/${v_dbvg}/LVDbSys2"
      v_inactive_part_no=1
   fi

   # Double check the inactive device. When exists check the size. It should be at least 30G
   # Only do this when backups are enabled
   if [ -b "${v_inactive_lvm_name}" ] 
   then
      local -i v_inac_lvm_size=`${v_lvm} lvdisplay --ignorelockingfailure --columns --noheadings -o lv_size --units g ${v_inactive_lvm_name} 2>/dev/null \
                               | awk -F "." ' { print $1 } ' \
                               | sed -e "s/ //g"`

       if [ ${v_inac_lvm_size} -lt 30 ]
       then
          v_logger_msg="The inactive lvm (${v_inactive_lvm_name}) has a size smaller than 30GB. Backups will fail. Re-create it with proper size"
          imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
          v_checks_failed="CheckLvmSize"
          PrintErrorReturn noexit
          v_tmp_cnt=1
       fi

       local -i v_ac_lvm_size=`${v_lvm} lvdisplay --ignorelockingfailure --columns --noheadings -o lv_size --units g ${v_active_lvm_name} 2>/dev/null \
                               | awk -F "." ' { print $1 } ' \
                               | sed -e "s/ //g"`

       if [ ${v_inac_lvm_size} -lt ${v_ac_lvm_size} ]
       then
          v_logger_msg="Inactive lvm (${v_inactive_lvm_name}) (${v_inac_lvm_size}G) not equal to active lvm ${v_active_lvm_name} (${v_ac_lvm_size}G). Backups will fail. Re-create it with proper size"
          imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
          v_checks_failed="CheckLvmSize"
          PrintErrorReturn noexit
          v_tmp_cnt=1
       fi

       if [ ${v_tmp_cnt} -eq 0 ]
       then
          v_logger_msg="PASS: Lvm size"
          PrintMsg nolog info "${v_logger_msg}"
       fi
   fi

   v_logger_msg="Leaving ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
}

#--------------------------------------------------------------------------------
# PROCEDURE    : CheckLvmActive
# INPUT        :
# DESCRIPTION  : Only Sys1 or Sys2 should be mounted
#------------------------------------------------------------------------------
CheckLvmActive()
{
   v_logger_msg="Entering ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   local v_lvm="/sbin/lvm"
   local v_dbvg="VGExaDb"
   local v_imageinfo="/usr/local/bin/imageinfo"
   local v_active_lvm_name=`${v_imageinfo} --system-partition 2>/dev/null`
   local -i v_tmp_cnt=1

   if [ "${v_active_lvm_name}" == "/dev/mapper/VGExaDb-LVDbSys1" ]
   then
      v_tmp_cnt=`/bin/mount -l | grep /dev/mapper/VGExaDb-LVDbSys2 | wc -l`
   fi

   if [ "${v_active_lvm_name}" == "/dev/mapper/VGExaDb-LVDbSys2" ]
   then
      v_tmp_cnt=`/bin/mount -l | grep /dev/mapper/VGExaDb-LVDbSys1 | wc -l`
   fi

   if [ ${v_tmp_cnt} -eq 0 ]
   then
      v_logger_msg="PASS: Inactive sys lvm's not mounted"
      PrintMsg nolog info "${v_logger_msg}"
   else
      v_checks_failed="CheckLvmActive"
      PrintErrorReturn noexit
   fi
   
   v_logger_msg="Leaving ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
}

#--------------------------------------------------------------------------------
# PROCEDURE    : is_second_arg_equal_or_bigger
# INPUT        :
#              :
# DESCRIPTION  :
#------------------------------------------------------------------------------
is_second_arg_equal_or_bigger ()
{
  # Remove any non digits from the both arguments
  local -i first=`echo $1 | perl -pne 's/\D//g'`
  local -i second=`echo $2 | perl -pne 's/\D//g'`

  [ $second -ge $first ] && return 0
  return 1
}


#--------------------------------------------------------------------------------
# PROCEDURE    : CheckFreeSpaceVG
# INPUT        :
# DESCRIPTION  : Check is there is enough free space if the vg (Sys1+Sys2+snapshot)
#------------------------------------------------------------------------------
CheckFreeSpaceVG()
{
   v_logger_msg="Entering ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   local v_lvm="/sbin/lvm"
   local v_dbvg="VGExaDb"
   local v_imageinfo="/usr/local/bin/imageinfo"
   local v_active_lvm_name=`${v_imageinfo} --system-partition 2>/dev/null`

   # Check free space in the Volume group
   vg_free_size_gb=`${v_lvm} vgdisplay --ignorelockingfailure --columns --noheadings -o vg_free --units g ${v_dbvg} 2>/dev/null`
   if [ -z "$vg_free_size_gb" ]
   then
      v_logger_msg="Unable to get total amount of free space in Volume group ${v_dbvg}"
      imageLogger_logMsg $lvmchecker $imageLogger_LOG_ERROR "${v_logger_msg}"
   fi
   vg_free_size_gb=`echo $vg_free_size_gb`
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "vg_free_size_gb: ${vg_free_size_gb}"

   vg_free_size_sc=`${v_lvm} vgdisplay --ignorelockingfailure --columns --noheadings -o vg_free --units s ${v_dbvg} 2>/dev/null`
   if [ -z "$vg_free_size_sc" ]
   then
      v_logger_msg="Unable to get total amount of free space in Volume group ${v_dbvg}"
      imageLogger_logMsg $lvmchecker $imageLogger_LOG_ERROR "${v_logger_msg}"
   fi
   vg_free_size_sc=`echo $vg_free_size_sc`
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "vg_free_size_sc: ${vg_free_size_sc}"
 
   v_logger_msg="Total amount of free space: $vg_free_size_gb ($vg_free_size_sc)"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   # Get size of root LVM partition
   root_size_gb=`${v_lvm} lvdisplay --ignorelockingfailure --columns --noheadings -o lv_size --units g ${v_active_lvm_name} 2>/dev/null`
   if [ -z "$root_size_gb" ]
   then
      v_logger_msg="Unable to get size of the root partition ${v_active_lvm_name}"
      imageLogger_logMsg $lvmchecker $imageLogger_LOG_ERROR "${v_logger_msg}"
   fi
   root_size_gb=`echo $root_size_gb`
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "root_size_gb: ${root_size_gb}"

   root_size_sc=`${v_lvm} lvdisplay --ignorelockingfailure --columns --noheadings -o lv_size --units s ${v_active_lvm_name} 2>/dev/null`
   if [ -z "$root_size_sc" ]
   then
      v_logger_msg="Unable to get size of the root partition ${v_active_lvm_name}"
      imageLogger_logMsg $lvmchecker $imageLogger_LOG_ERROR "${v_logger_msg}"
   fi
   root_size_sc=`echo $root_size_sc`
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "root_size_sc: ${root_size_sc}"

   # For system that have only LVDbSys1 or LVDbsys2 free space has to be equal or bigger than one root partition
   # this to make sure LVDbSys1 and LVDbSys2 both exist
   # plus the space for the snapshot which is 1GB  - but we use 2GB (comparison is in sector size where 1 sector=512 bytes)
   # lvm lvcreate -L 1G -s -n $LV_ROOT_SNAP $g_lv_root

   local -i v_lvm_cnt=`${v_lvdisplay} --ignorelockingfailure --columns --noheadings 2>/dev/null | grep  LVDbSys[1-2] | wc -l`

   if [ ${v_lvm_cnt} -lt 2 ]
   then
      # We assume LVDbSys2 is not created yet
      # So we require the space of LVDbSys1 + 2GB
      # Add 4194304 sectors (2GB) to the found root_size_sc

      # Remove the "S" and add the number
      root_size_sc=`echo $root_size_sc 2>/dev/null | sed 's/[^0-9]*//g'`
      root_size_sc="`expr ${root_size_sc} + 4194304 2>/dev/null`S"
      v_logger_msg="Adding 4194304 (2GB) sectors to root_size_sc. New value for root_size_sc: ${root_size_sc}"
      imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

      # If the free space in the VG is larger than the root size than we are good
      is_second_arg_equal_or_bigger $root_size_sc $vg_free_size_sc
      if [ $? -ne 0 ]
      then
         v_checks_failed="CheckFreeSpaceVG"
         PrintErrorReturn noexit
      else
         v_logger_msg="PASS: Enough free space found for snapshot"
         PrintMsg nolog info "${v_logger_msg}"
      fi
   else
      # We assume LVDbSys1 and LVDbSys2 exist and require only enough space for the 1GB snapshot - take 2GB
      # Remove the "S" from vg_free_size_sc
      vg_free_size_sc=`echo ${vg_free_size_sc} | sed 's/[^0-9]*//g'`
      if [ ${vg_free_size_sc} -lt 4194304 ]
      then
         v_checks_failed="CheckFreeSpaceVG"
         PrintErrorReturn noexit
      else
            v_logger_msg="PASS: Enough free space found for snapshot"
            PrintMsg nolog info "${v_logger_msg}"
      fi
   
   fi

   v_logger_msg="Leaving ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
}

#--------------------------------------------------------------------------------
# PROCEDURE    : CheckVGExa
# INPUT        :
# DESCRIPTION  : LVDbSys1 and LVDbSys2 should always reside in VGExa (CheckVGExa)
#------------------------------------------------------------------------------
CheckVGExa()
{
   v_logger_msg="Entering ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"


   local v_lvm_name=""
   local -i v_tmp_cnt=0
   local -i c=0
  
   for v_lvm_name in `/usr/sbin/lvdisplay --ignorelockingfailure --columns --noheadings -o lv_name | grep " LVDbSys[1-2] " | sed -e "s/ //g"`
   do
      v_tmp_cnt=`/usr/sbin/lvdisplay --ignorelockingfailure --columns --noheadings -o vg_name /dev/VGExaDb/${v_lvm_name} | wc -l`
      if [ ${v_tmp_cnt} -eq 0 ]
      then
         c=`expr ${c} + 1`
      fi

      if [ ${c} -gt 0 ]
      then
         v_checks_failed="CheckVGExa"
         PrintErrorReturn noexit
      else
         v_logger_msg="PASS:  ${v_lvm_name} should reside in VGExa"
         PrintMsg nolog info "${v_logger_msg}"
      fi
   done

   v_logger_msg="Leaving ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
}

#--------------------------------------------------------------------------------
# PROCEDURE    : CheckIsLvmSys
# INPUT        :
# DESCRIPTION  : Check is the system is lvm enabled
#------------------------------------------------------------------------------
CheckIsLvmSys()
{
   v_logger_msg="Entering ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"

   local v_lvm_install=`awk -F= '/^lvm=/ {print $2}' /opt/oracle.cellos/ORACLE_CELL_OS_IS_SETUP 2>/dev/null`

   if [ "${v_lvm_install}" != "yes" ]
   then
      v_checks_failed="CheckIsLvmSys"
      PrintErrorReturn exit
   else
      v_logger_msg="PASS: This is an lvm system "
      PrintMsg nolog info "${v_logger_msg}"
   fi

   v_logger_msg="Leaving ${FUNCNAME} $1 $2 $3"
   imageLogger_logMsg $lvmchecker $imageLogger_LOG_INFO "${v_logger_msg}"
}

#--------------------------------------------------------------------------------
# MAIN         : Start of script
# INPUT        :
# DESCRIPTION  : Support only Linux
#------------------------------------------------------------------------------
[[ $(uname) != Linux ]] && {
  PrintMsg nolog info
  PrintMsg nolog info "This script only support on Linux, exiting..."
  PrintMsg nolog info
  exit 1
}

#--------------------------------------------------------------------------------
# ENV SETTINGS :
# INPUT        :
# DESCRIPTION  : Default env settings
#------------------------------------------------------------------------------
LANG="en_US.UTF-8"
LC_ALL=C
export LANG LC_ALL


#--------------------------------------------------------------------------------
# PARAMS       :
# INPUT        :
# DESCRIPTION  : Default param values
#------------------------------------------------------------------------------
v_checks_failed=""
v_action=""
v_myname="checkvg.sh"
v_logfile=checkvg
v_timestamp=`date '+%d%m%y_%H%M%S'`
v_runid=`date '+%d%m%y%H%M%S'`
v_arguments="$@"
v_imagelogger="/opt/oracle.cellos/imageLogger"
v_error_cnt=0
v_myname_version="1.01"

# Checks we implement according to 19068757 
#------------------------------------------------------------------------------
# 1.  A system may not be an lvm enabled system - then the check does not apply  (CheckIsLvmSys)
# 1b. LVDbSys1 and LVDbSys2 should always reside in VGExa (CheckVGExa)
# 2.  The LVDbSys2 lvm can exist - but does not have to (CheckMinLvm)
# 3.  If LVDbSys1 lvm is active - LVDbSys2 can exist, then LVDbSys2 and LVDbSys1  should be the same size (CheckLvmSize) 
# 4.  If LVDbSys2 lvm is active - LVDbSys1 exists - then LVDbSys2 and LVDbSys1  should be the same size (CheckLvmSize)
# 5.  LVDbSys2 and LVDbSys1 should not be mounted at the same time  (CheckLvmActive)
# 6.  If LVDbSys2 does not exist - then enough free space is required to create  it (with size of LVDbSys1 ) (CheckFreeSpaceVG) 
# 7.  On top of (6) enough free space should exist for the 1GB snapshot  (but take 2GB to be sure)  (CheckFreeSpaceVG) 
# 8.  Customers may decide to do backups and not use lvms for that (No check)
#------------------------------------------------------------------------------

# Check number of arguments. No argument assumes we want to run allchecks
# This is a quick implementation - should be replaced by getopts later 
if [ $# -lt 1 ]
then
   v_action="checkall"    
else
   v_action="$1"    
fi

if [ ! -f ${v_imagelogger} ]
then
   echo "Required file (${v_imagelogger}) not in place, exiting"
   exit 1
else
   # v_logger_msg="Initializing logfile /var/log/cellos/${v_logfile}.log"
   # echo "  (*) `date '+%F %T'`: ${v_logger_msg}"
   . ${v_imagelogger}
   imageLogger_init lvmchecker -name=${v_logfile} -notrace -silent
   # v_logfile="/var/log/cellos/${v_logfile}.log"
   v_helpers_init=Yes
fi

InitLog

case "${v_action}" in
     islvm)
           CheckIsLvmSys
           ;;
     vgexa)
           CheckVGExa
           ;;
     lvmmin)
           CheckMinLvm
           ;;
     lvmsize)
           CheckLvmSize
           ;;
     lvmactive)
           CheckLvmActive
           ;;
     vgfree)
           CheckFreeSpaceVG
           ;;
   checkall)
           # exec steps for completion (relink etc)
           ExecAllChecks
           ;;
          *)
           usage
           exit 1
           ;;
esac

# For more errors than 0 the return value will be 1
if [ ${v_error_cnt} -gt 0 ]
then
   exit 1
else
   exit 0
fi


